#include <openssl/ssl.h>
#include <openssl/err.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

  int sock;
  struct sockaddr_in sock_addr;
  char data[1024+1];
  int len;
  SSL_CTX *ssl_ctx;
  SSL *tls;
  char str[10],*name;
  X509 *cert;

/* 
  Vytvoreni socketu a pripojen k serveru
 */
int tcp_connect(char *ip)
{
   int err, sd;
   struct sockaddr_in sa;

   //vytvoreni socketu
   sd = socket(AF_INET, SOCK_STREAM, 0);

   memset(&sa, '\0', sizeof(sa));
   sa.sin_family = AF_INET;
   sa.sin_port = htons(23987);

   //preklad ip adresy   
   struct hostent *phe;
   if((phe = gethostbyname(ip)) != NULL)
     memcpy((char *)&sa.sin_addr, phe->h_addr, phe->h_length);
   else if((sa.sin_addr.s_addr = inet_addr(ip)) == INADDR_NONE) {
     printf("Nelze ziskat ip adresu ani jmeno serveru\n");
     exit(1);
   }

   //pripojen k serveru
   err = connect(sd, (struct sockaddr *) & sa, sizeof(sa));
   if (err < 0) {
      printf("Nelze se pripojit\n");
      exit(1);
   }

   return sd;
} //tcp_connect


  /*
    TLS klient realizovany pomoci OpenSSL.
    Klient se spousti s jedinym parametrem ip nebo jmeno serveru.
  */
  int main(int argc, char** args)
  {
   if(argc != 2) {
     printf("Program se spousti s jedinym parametrem.\nZadejte jmeno serveru nebo IP adresu.\n");
     return 5;
   }   

    SSL_load_error_strings();
    SSLeay_add_ssl_algorithms();
    
    //bude to TLS klient
    if(!(ssl_ctx = SSL_CTX_new(TLSv1_client_method()))) {
      ERR_print_errors_fp(stderr);
      return 1;
    }

    //pripojeni
    sock = tcp_connect(args[1]);

    //priprava na handshake
    if(!(tls = SSL_new(ssl_ctx))) {
      ERR_print_errors_fp(stderr);
      return 1;
    }
    //spojeni socketu se tls
    if(!SSL_set_fd(tls, sock)) {
      ERR_print_errors_fp(stderr);
      return 1;
    }
    //provedeni handshaku
    if(SSL_connect(tls) != 1) {
      ERR_print_errors_fp(stderr);
      return 1;
    }

    //info o serveru  
    printf("Pouzita sifra %s\n", SSL_get_cipher(tls));
    if(!(cert = SSL_get_peer_certificate(tls))) {
      printf("Server neposlal certifikat\n");
      ERR_print_errors_fp(stderr);
    }
    else {
      if((name = X509_NAME_oneline(X509_get_subject_name(cert),0,0))) {
        printf("Certifikat serveru: %s\n",name);
        free(name);
      }
      X509_free(cert);
    }
    
    //odeslu zpravu
    strcpy(str, "Tajna zprava\r\n");
    len = strlen(str);
    printf("Odesilam %d bytu: %s ", len, str);        
    if(SSL_write(tls, str,len) != len) {
      ERR_print_errors_fp(stderr);
      return 1;
    }

    //prijmu zpravu
    if ((len = SSL_read(tls, data, 1024)) <= 0) {
      ERR_print_errors_fp(stderr);
      return 1;
    }
    data[len]=0;
    //vypisu
    printf("Prijmuto %d bytu: %s", len, data);        

    //uklid
    SSL_free(tls);
    close(sock);
  } //main
